#include <windows.h>
#include <stdio.h>
#include <lmerr.h>

#define RTN_OK 0
#define RTN_USAGE 1
#define RTN_ERROR 13

void DisplayErrorText(DWORD dwLastError);

int main(int argc,char *argv[])
{
    if(argc != 2)
	{
        fprintf(stderr,"Usage: %s <error number>\n", argv[0]);
        return RTN_USAGE;
    }

    DisplayErrorText(atoi(argv[1]));

    return RTN_OK;
}

void DisplayErrorText(DWORD dwLastError)
{
    HMODULE hModule = NULL; // default to system source
    LPSTR MessageBuffer;
    DWORD dwBufferLength;

    DWORD dwFormatFlags = FORMAT_MESSAGE_ALLOCATE_BUFFER |
        FORMAT_MESSAGE_IGNORE_INSERTS |
        FORMAT_MESSAGE_FROM_SYSTEM ;

	// http://msdn.microsoft.com/en-us/library/aa370674%28VS.85%29.aspx
    // If dwLastError is in the network range, 
    //  load the message source.
	// NERR_BASE = 1200
    if(dwLastError >= NERR_BASE && dwLastError <= MAX_NERR)
	{
        hModule = LoadLibraryEx(
            TEXT("netmsg.dll"),
            NULL,
            LOAD_LIBRARY_AS_DATAFILE
            );

        if(hModule != NULL)
            dwFormatFlags |= FORMAT_MESSAGE_FROM_HMODULE;
    }

    // Call FormatMessage() to allow for message 
    //  text to be acquired from the system 
    //  or from the supplied module handle.
    if(dwBufferLength = FormatMessageA(
        dwFormatFlags,
        hModule, // module to get message from (NULL == system)
        dwLastError,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // default language
        (LPSTR) &MessageBuffer,
        0,
        NULL
        ))
    {
        DWORD dwBytesWritten;

        // Output message string on stderr.
        WriteFile(
            GetStdHandle(STD_ERROR_HANDLE),
            MessageBuffer,
            dwBufferLength,
            &dwBytesWritten,
            NULL
            );

        // Free the buffer allocated by the system.
        LocalFree(MessageBuffer);
    }

    // If we loaded a message source, unload it.
    if(hModule != NULL)
        FreeLibrary(hModule);
}
